//

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//


#ifndef Fl_GDI_Printer_H
#define Fl_GDI_Printer_H
#include "Fl_Printer.H"
#include <windows.h>
#include <FL/x.H>
#include <commdlg.h>


extern FL_EXPORT Fl_Color fl_color_;
extern FL_EXPORT int fl_size_;
extern FL_EXPORT int fl_font_;
extern FL_EXPORT HDC fl_gc;

/*
struct Fl_Ext{
    int WOx, VEx, WEx, VOx;
    int WOy, VEy, WEy, VOy;
};







class FL_EXPORT Fl_Gdi_Settings{

  public:
  PRINTDLG pd;

  HDC gc(){return pd.hDC;};

  DEVMODE * mode() {return (DEVMODE *) pd.hDevMode;};
  void pages(int min_page, int max_page, int first=1, int last = 0);
  int max_page();
  void max_page(int p);
  int format();
  void format(int f);
  int orientation();
  void orientation(int o);

  Fl_Gdi_Settings();
  ~Fl_Gdi_Settings();

};

*/
struct Fl_Brush {
    HBRUSH brush;
    unsigned short usage;
    Fl_XMap* backref;
};


class FL_EXPORT Fl_GDI_Printer: public Fl_Printer{

  HPEN tmppen;
  HBRUSH tmpbrush;
  HPEN savepen;

  Fl_Brush brushes[FL_N_BRUSH];
  Fl_XMap fl_xmap[256];
  Fl_XMap xmap;
  
  HDC gc_;
  DEVMODE * mode_;
//  int delete_mode_;
  long ix, iy, ox, oy;
  long WOx, VEx, WEx, VOx;
  long WOy, VEy, WEy, VOy;
  int style_;
  int width_;
  char * dashes_;

  long max_res_;
  uchar * mask;
  void sty(int style, int width, char *dashes, int vex = 1);
  
  
  void set_page(int page);
  void set_subpixel();
  void set_normal();
 
protected:

public:
    Fl_Output_Device * set_current();
//    void delete_mode(int i){delete_mode_=i;};

  void color(Fl_Color c){
      sty(0,0,0);
      Fl_Output_Device::color(c);
      sty(style_, width_, dashes_);
  };

  void color(uchar r, uchar g, uchar b){
      sty(0,0,0);
      Fl_Output_Device::color(r, g, b);
      sty(style_, width_, dashes_);
  }


    void draw(Fl_Widget * w);
    void page(double pw, double ph, int media = 0);
    void page(int format);
    //void page();
    void line_style(int style, int width, char * dashes);
    //have to fill properly the area
    void rectf(int x, int y, int w, int h);
    void point(int x, int y);
    void arc(int x, int y, int w, int h, double a1, double a2);
    void pie(int x, int y, int w, int h, double a1, double a2);

    // unfortunately must be re-implemented
    void xyline(int x, int y, int x1);
    void yxline(int x, int y, int y1);
    void xyline(int x, int y, int x1, int y2);
    void xyline(int x, int y, int x1, int y2, int x3);
    void yxline(int x, int y, int y1, int x2);
    void yxline(int x, int y, int y1, int x2, int y3);
    void line(int x, int y, int x1, int y1);
    void line(int x, int y, int x1, int y1, int x2, int y2);

    //  these need to be redefined to support more precise (subpixel)  path ("double") drawings
    void transformed_vertex(double x, double y);
    void vertex(double x, double y);
    void circle(double x, double y, double r);
    void end_line();
    void end_polygon();
    void end_loop();
    void end_complex_polygon();

    // other "double" drawing functions use above, do not need re-implementation

    // Some clipping primitives must be re-implemented as core gdi clipping functions work in device-coordinates, not "logical" (in MS terminology)
    void push_clip(int x, int y, int w, int h);
    int not_clipped(int x, int y, int w, int h);
    int clip_box(int x, int y, int w, int h, int &X, int &Y, int &W, int &H);

    void margins(double left, double top, double right, double bottom);

       // placement of drawings on the page
    void place(double x, double y,  double tx, double ty,  double scale = 1);
    //void place(double x, double y, double w, double h, double tx, double ty, double tw, double th, int align = FL_ALIGN_CENTER);

 
  // Following is a hack  -  need to be uncached for different gc.
  // note: masking do not seems to work because printer drivers usualy can not de-compose background in BitBlt
  // Maybe AlphaBlend() (MaskBlt?) would  work, but this probably needs to be done in the core.
  // AlphaBlend (and MaskBlt) are implemented in Win98 and above, not for Win95.
  //  void draw_Pixmap(Fl_Pixmap * pxm,int XP, int YP, int WP, int HP, int cx, int cy);
  //  void draw_RGB(Fl_RGB_Image * rgb,int XP, int YP, int WP, int HP, int cx, int cy);
  //  void draw_Bitmap(Fl_Bitmap * bmp,int XP, int YP, int WP, int HP, int cx, int cy);

  Fl_GDI_Printer(HDC gc, DEVMODE * mode);
  ~Fl_GDI_Printer();
};


//Fl_GDI_Printer * fl_gdi_printer_chooser(Fl_Gdi_Settings * settings=0);

#endif
