#ifndef Fl_Output_Device_H
#define Fl_Output_Device_H
#include <FL/Enumerations.H>
#include <FL/Fl_Widget.H>
#ifdef WIN32
#undef USE_XFT
#endif
#ifdef USE_XFT
#include <X11/Xft/Xft.h>
#endif
inline void fl_draw(Fl_Widget * w){w->redraw(); w->draw();};

//#include <FL/Fl_Image.H>
//#include <FL/Fl_Pixmap.H>
//#include <FL/Fl_Bitmap.H>

class Fl_Image;
class Fl_RGB_Image;
class Fl_Pixmap;
class Fl_Bitmap;

extern FL_EXPORT Fl_Color fl_color_;
extern FL_EXPORT int fl_size_;
extern FL_EXPORT int fl_font_;

#ifdef WIN32
#define FL_N_BRUSH 16
#endif

typedef void (*Fl_Draw_Image_Cb)(void* ,int,int,int,uchar*);




class FL_EXPORT Fl_Output_Device{
static long dev_index_count;

protected:
  int type_; // last byte is for screen devices;

 // following is unique for every device instance,  it is used for (un)caching to indicate if recent
 // cache belongs to that device
 long dev_index;

 // uncaching must be different for each device type but can not be virtual as the devcice instance my not exist
 // when uncaching has to be performed
 static void uncache_RGB(Fl_Image *);
 static void uncache_Bitmap(Fl_Image *);
 static void uncache_Pixmap(Fl_Image *);

public:
  enum {SCREEN =
#ifdef WIN32
    1
#elif defined(__APPLE__)
    2
#else
    0
#endif
, PRINTER = 0xFF00
};
  int type(){return type_;}; // <256 for screen devices

  virtual   void color(Fl_Color color);
  virtual   void color(uchar r, uchar g, uchar b);
  virtual Fl_Color color(){return fl_color_;};

  virtual   void push_clip(int x, int y, int w, int h);


  virtual   void push_no_clip();
  virtual   void pop_clip();
  virtual   int not_clipped(int x, int y, int w, int h);
  virtual   int clip_box(int _x, int _y, int _w, int _h, int& x, int& y, int& w, int& h);


  virtual   void point(int x, int y);
  virtual   void line_style(int style, int width=0, char* dashes=0);

  virtual   void rect(int x, int y, int w, int h);
  virtual   void rectf(int x, int y, int w, int h);

  virtual   void line(int x1, int y1, int x2, int y2);
  virtual   void line(int x1, int y1, int x2, int y2, int x3, int y3);

  virtual   void loop(int x1, int y1, int x2 ,int y2, int x3, int y3);
  virtual   void loop(int x1, int y1, int x2 ,int y2, int x3, int y3, int x4, int y4);

  virtual   void polygon(int x1, int y1, int x2 ,int y2, int x3, int y3);
  virtual   void polygon(int x1 ,int y1, int x2, int y2, int x3, int y3, int x4, int y4);

  virtual   void xyline(int x, int y, int x1);
  virtual   void xyline(int x, int y, int x1, int y2);
  virtual   void xyline(int x, int y, int x1, int y2, int x3);

  virtual   void yxline(int x, int y, int y1);
  virtual   void yxline(int x, int y, int y1, int x2);
  virtual   void yxline(int x, int y, int y1, int x2, int y3);

  virtual   void arc(int x, int y, int w, int h, double a1, double a2);
  virtual   void pie(int x, int y, int w, int h, double a1, double a2);

  virtual   void begin_points();
  virtual   void begin_line();
  virtual   void begin_loop();
  virtual   void begin_polygon();
  virtual   void vertex(double x, double y);
  virtual   void curve(double x, double y, double x1, double y1, double x2, double y2, double x3, double y3);
  virtual   void arc(double x, double y, double r, double start, double a);
  virtual   void circle(double x, double y, double r);
  virtual   void end_points();
  virtual   void end_line();
  virtual   void end_loop();
  virtual   void end_polygon();
  virtual   void begin_complex_polygon();
  virtual   void gap();
  virtual   void end_complex_polygon();
  virtual   void transformed_vertex(double x, double y);

  virtual   void font(int face, int size);
  virtual   void font(Fl_Font, Fl_Fontsize, int);
  virtual   int font(){return fl_font_;};
  virtual   int size(){return fl_size_;};
  virtual   int height();
  virtual   double width(const char* s);
  virtual   double width(const char* s, int n);
#ifdef USE_XFT
  virtual   double width (FcChar32 *str, int n);
  virtual   double width(unsigned int c);
  virtual void draw(int, const char*, int, int,int);
  virtual void draw(const char*, int, float, float);
  virtual void draw(int, const char*, int, int);
#endif
#ifdef __APPLE__
  double width(unsigned int);
  double width(const unsigned short int*, int);
  double width(int);
  void draw(int, const char*, int, int);
  void draw(const char*, int, float, float);
  void draw(int, const char*, int, int, int);
#endif
#ifdef WIN32
  virtual   double width(unsigned int c);
  virtual void draw(int, const char*, int, int);
  virtual void draw(const char*, int, float, float);
  virtual void draw(int, const char*, int, int,int);
#else
  virtual   double width(uchar c);
#endif
  virtual   int descent();
  virtual   void draw(const char* s, int x, int y);
  virtual   void draw(const char* s, int n, int x, int y);
  virtual   void transformed_draw(const char* s, int n, double x, double y){draw(s,n,(int)(x+.5),(int)(y+.5));};
  virtual   void transformed_draw(const char* s, double x, double y){draw(s,(int)(x+.5),(int)(y+.5));};



  virtual   void draw_image(const uchar*, int,int,int,int, int delta=3, int ldelta=0);
  virtual   void draw_image_mono(const uchar*, int,int,int,int, int delta=1, int ld=0);
  virtual   void draw_image(Fl_Draw_Image_Cb, void*, int,int,int,int, int delta=3);
  virtual   void draw_image_mono(Fl_Draw_Image_Cb, void*, int,int,int,int, int delta=1);
  virtual   void rectf(int x, int y, int w, int h, uchar r, uchar g, uchar b);
  virtual   void draw(Fl_Pixmap * pxm,int XP, int YP, int WP, int HP, int cx, int cy);
  virtual   void draw(Fl_RGB_Image * rgb,int XP, int YP, int WP, int HP, int cx, int cy);
  virtual   void draw(Fl_Bitmap * bmp,int XP, int YP, int WP, int HP, int cx, int cy);

  virtual Fl_Output_Device * set_current();
  Fl_Output_Device * current(Fl_Output_Device * dev){return dev->set_current();};
  Fl_Output_Device * current();

  virtual   void draw(Fl_Widget *w){Fl_Output_Device * c = set_current(); w->redraw(); fl_draw(w); c->set_current();};


  virtual   ~Fl_Output_Device(){};
  Fl_Output_Device(){type_ = SCREEN; dev_index = dev_index_count++;};


};

#ifdef WIN32
//typedef Fl_Output_Device Fl_Gdi;
#endif

extern FL_EXPORT Fl_Output_Device fltk;
extern FL_EXPORT Fl_Output_Device *fl;

#endif
