//
// "$Id: fl_draw.cxx,v 1.6.2.4.2.12 2002/05/16 12:47:43 easysw Exp $"
//
// Label drawing code for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2002 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "fltk-bugs@fltk.org".
//

// Implementation of fl_draw(const char*,int,int,int,int,Fl_Align)
// Used to draw all the labels and text, this routine:
// Word wraps the labels to fit into their bounding box.
// Breaks them into lines at the newlines.
// Expands all unprintable characters to ^X or \nnn notation
// Aligns them against the inside of the box.

// #define min(a,b) ((a)<(b)?(a):(b))
#include <FL/fl_draw.H>
#include <FL/Fl_Image.H>

#include "flstring.h"
#include <ctype.h>


#define MAXBUF 1024

#include <FL/Fl_Output_Device.H>
//#include <FL/Fl_Group.H>
#include <FL/Fl_Pixmap.H>
#include <FL/Fl_Bitmap.H>

Fl_Output_Device fltk;
Fl_Output_Device * fl=&fltk;



long Fl_Output_Device::dev_index_count=0;

#include "fl_draw_generic.cxx"

#include "Fl_Printer.cxx"
//#include "Fl_PS_Printer.cxx"

#ifdef WIN32

#include <FL/x.H>
extern FL_EXPORT void pop_xmaps();
extern FL_EXPORT void push_xmaps();
#endif


#include <FL/Fl.H>


Fl_Output_Device * Fl_Output_Device::set_current(){
  if(fl == this) return this;


  if(fl == &fltk){
    Fl::flush();
#ifdef WIN32
    push_xmaps();
#endif
  }
  Fl_Output_Device * c = fl;
  fl = this;
#ifdef WIN32
  if(fl == &fltk)
    pop_xmaps(); // also sets fl_gc back
#endif
   return c;
};
Fl_Output_Device * Fl_Output_Device::current(){ return fl;};


/*
// for now i just include printer classes, it will be in separate libs later

#include "Fl_PS_Printer.cxx"

#ifdef WIN32
  #include "Fl_GDI_Printer.cxx"
#endif

*/




void Fl_Bitmap::draw(int X, int Y, int W, int H, int cx, int cy){
	fl->draw(this, X, Y, W, H, cx, cy);
}

void Fl_Pixmap::draw(int X, int Y, int W, int H, int cx, int cy){
	fl->draw(this, X, Y, W, H, cx, cy);
}

void Fl_RGB_Image::draw(int X, int Y, int W, int H, int cx, int cy){
	fl->draw(this, X, Y, W, H, cx, cy);
}

void fl_draw(const char* s, int x, int y){fl->draw(s,x,y);};
void fl_draw(const char* s, int n, int x, int y){fl->draw(s,n,x,y);};
void fl_draw(const char* s, int n, float x, float y){fl->draw(s,n,x,y);};
//void fl_draw(Fl_Widget * w){ w->redraw();  w->draw(); }

 void	fl_color(Fl_Color c){fl->color(c);}; // select indexed color
 void fl_color(int c) {fl_color((Fl_Color)c);} // for back compatability
 void	fl_color(uchar r, uchar g , uchar b){fl->color(r,g,b);}; // select actual color
 Fl_Color fl_color(){return fl->color();}

 void fl_push_clip(int x, int y, int w, int h){fl->push_clip(x,y,w,h);};
 void fl_push_no_clip(){fl->push_no_clip();};
 void fl_pop_clip(){fl->pop_clip();};
 int fl_not_clipped(int x, int y, int w, int h){return fl->not_clipped(x,y,w,h);};
// int fl_not_clipped(double x, double y, double w, double h){return fl->not_clipped(x,y,w,h);};
 int fl_clip_box(int _x, int _y, int _w, int _h, int& x, int& y, int& w, int& h){return fl->clip_box(_x, _y, _w, _h, x, y, w, h);};
 void fl_point(int x, int y){fl->point(x,y);};

// line type:
 void fl_line_style(int style, int width, char* dashes){fl->line_style(style,width,dashes);};
 void fl_rect(int x, int y, int w, int h){fl->rect(x,y,w,h);};
 void fl_rectf(int x, int y, int w, int h){fl->rectf(x,y,w,h);};

 void fl_rect(int x, int y, int w, int h, Fl_Color c) {fl_color(c); fl_rect(x,y,w,h);}
 void fl_rectf(int x, int y, int w, int h, Fl_Color c) {fl_color(c); fl_rectf(x,y,w,h);}

 void fl_line(int x1, int y1, int x2, int y2){fl->line(x1, y1, x2, y2);};
 void fl_line(int x1, int y1, int x2, int y2, int x3, int y3){fl->line(x1, y1, x2, y2, x3, y3);};

// closed line segments:
 void fl_loop(int x1, int y1, int x2 ,int y2, int x3, int y3){fl->loop(x1, y1, x2, y2, x3, y3);};
 void fl_loop(int x1, int y1, int x2 ,int y2, int x3, int y3, int x4, int y4){fl->loop(x1, y1, x2, y2, x3, y3, x4, y4);};

// filled polygons
 void fl_polygon(int x1, int y1, int x2 ,int y2, int x3, int y3){fl->polygon(x1, y1, x2, y2, x3, y3);};
 void fl_polygon(int x1 ,int y1, int x2, int y2, int x3, int y3, int x4, int y4){fl->polygon(x1, y1, x2, y2, x3, y3, x4, y4);};

// draw rectilinear lines, horizontal segment first:
 void fl_xyline(int x, int y, int x1){fl->xyline(x,y,x1);};
 void fl_xyline(int x, int y, int x1, int y2){fl->xyline(x,y,x1,y2);};
 void fl_xyline(int x, int y, int x1, int y2, int x3){fl->xyline(x,y,x1,y2,x3);};

// draw rectilinear lines, vertical segment first:
 void fl_yxline(int x, int y, int y1){fl->yxline(x,y,y1);};
 void fl_yxline(int x, int y, int y1, int x2){fl->yxline(x,y,y1,x2);};
 void fl_yxline(int x, int y, int y1, int x2, int y3){fl->yxline(x,y,y1,x2,y3);};

 void fl_arc(int x, int y, int w, int h, double a1, double a2){fl->arc(x,y,w,h,a1,a2);};
 void fl_pie(int x, int y, int w, int h, double a1, double a2){fl->pie(x,y,w,h,a1,a2);};

 void fl_begin_points(){fl->begin_points();};
 void fl_begin_line(){fl->begin_line();};
 void fl_begin_loop(){fl->begin_loop();};
 void fl_begin_polygon(){fl->begin_polygon();};
 void fl_vertex(double x, double y){fl->vertex(x,y);};
 void fl_curve(double x, double y, double x1, double y1, double x2, double y2, double x3, double y3){fl->curve(x,y,x1,y1,x2,y2,x3,y3);};
 void fl_arc(double x, double y, double r, double start, double a){fl->arc(x,y,r,start,a);};
 void fl_circle(double x, double y, double r){fl->circle(x,y,r);};
 void fl_end_points(){fl->end_points();};
 void fl_end_line(){fl->end_line();};
 void fl_end_loop(){fl->end_loop();};
 void fl_end_polygon(){fl->end_polygon();};
 void fl_begin_complex_polygon(){fl->begin_complex_polygon();};
 void fl_gap(){fl->gap();};
 void fl_end_complex_polygon(){fl->end_complex_polygon();};
 void fl_transformed_vertex(double x, double y){fl->transformed_vertex(x,y);};
 void fl_font(int face, int size){fl->font(face,size);};
 int fl_font() {return fl->font();};
 int fl_size() {return fl->size();};
 int   fl_height(){return fl->height();};	// using "size" should work ok

 int   fl_descent(){return fl->descent();};
 double fl_width(const char* s){return fl->width(s);};
 double fl_width(const char* s, int n){return fl->width(s,n);};
 double fl_width(uchar c){return fl->width(c);};
 void fl_draw_image(const uchar* im, int x,int y,int w,int h, int delta, int ldelta){fl->draw_image(im,x,y,w,h,delta,ldelta);};
 void fl_draw_image_mono(const uchar* im, int x,int y,int w,int h, int delta, int ld){fl->draw_image_mono(im,x,y,w,h,delta,ld);};
typedef void (*Fl_Draw_Image_Cb)(void* ,int,int,int,uchar*);
 void fl_draw_image(Fl_Draw_Image_Cb cb, void* im, int x,int y,int w,int h, int delta){fl->draw_image(cb, im, x,y,w,h,delta);};
 void fl_draw_image_mono(Fl_Draw_Image_Cb cb, void* im, int x,int y,int w,int h, int delta){fl->draw_image(cb, im, x,y,w,h,delta);};
 void fl_rectf(int x, int y, int w, int h, uchar r, uchar g, uchar b){fl->rectf(x,y,w,h,r,g,b);};

